@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 MySQL AB
+/* Copyright (c) 2000, 2010, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,12 +11,15 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 
 /* Functions to handle keys and fields in forms */
 
-#include "mysql_priv.h"
+#include "sql_priv.h"
+#include "unireg.h"                     // REQUIRED: by includes later
+#include "key.h"                                // key_rec_cmp
+#include "field.h"                              // Field
 
 /*
   Search after a key that starts with 'field'
@@ -349,10 +352,37 @@ void key_unpack(String *to,TABLE *table,uint idx)
     }
     if ((field=key_part->field))
     {
+      CHARSET_INFO *cs= field->charset();
       field->val_str(&tmp);
+      /*
+        For BINARY(N) strip trailing zeroes to make
+        the error message nice-looking
+      */
+      if (field->binary() &&  field->type() == MYSQL_TYPE_STRING && tmp.length())
+      {
+        const char *tmp_end= tmp.ptr() + tmp.length();
+        while (tmp_end > tmp.ptr() && !*--tmp_end) ;
+        tmp.length(tmp_end - tmp.ptr() + 1);
+      }
+      if (cs->mbmaxlen > 1 && (key_part->key_part_flag & HA_PART_KEY_SEG))
+      {
+        /* 
+          Prefix key, multi-byte charset. 
+          For the columns of type CHAR(N), the above val_str() 
+          call will return exactly "key_part->length" bytes, 
+          which can break a multi-byte characters in the middle. 
+          Align, returning not more than "char_length" characters. 
+        */
+        uint charpos, char_length= key_part->length / cs->mbmaxlen;
+        if ((charpos= my_charpos(cs, tmp.ptr(),
+                                 tmp.ptr() + tmp.length(),
+                                 char_length)) < tmp.length())
+          tmp.length(charpos);
+      }
       if (key_part->length < field->pack_length())
 	tmp.length(min(tmp.length(),key_part->length));
-      to->append(tmp);
+      ErrConvString err(&tmp);
+      to->append(err.ptr());
     }
     else
       to->append(STRING_WITH_LEN("???"));
