@@ -1,5 +1,4 @@
-/*
-   Copyright (c) 2000, 2010, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,17 +11,13 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* open a MyISAM MERGE table */
 
 #include "myrg_def.h"
 #include <stddef.h>
 #include <errno.h>
-#ifdef VMS
-#include "mrg_static.c"
-#endif
 
 /*
 	open a MyISAM MERGE table
@@ -52,9 +47,10 @@ MYRG_INFO *myrg_open(const char *name, int mode, int handle_locking)
   DBUG_ENTER("myrg_open");
 
   bzero((char*) &file,sizeof(file));
-  if ((fd=my_open(fn_format(name_buff,name,"",MYRG_NAME_EXT,
-                            MY_UNPACK_FILENAME|MY_APPEND_EXT),
-		  O_RDONLY | O_SHARE,MYF(0))) < 0)
+  if ((fd= mysql_file_open(rg_key_file_MRG,
+                           fn_format(name_buff, name, "", MYRG_NAME_EXT,
+                                     MY_UNPACK_FILENAME|MY_APPEND_EXT),
+                           O_RDONLY | O_SHARE, MYF(0))) < 0)
     goto err;
   errpos=1;
   if (init_io_cache(&file, fd, 4*IO_SIZE, READ_CACHE, 0, 0,
@@ -81,7 +77,7 @@ MYRG_INFO *myrg_open(const char *name, int mode, int handle_locking)
     {
       if (!strncmp(buff+1,"INSERT_METHOD=",14))
       {			/* Lookup insert method */
-	int tmp=find_type(buff+15,&merge_insert_method,2);
+	int tmp= find_type(buff + 15, &merge_insert_method, FIND_TYPE_BASIC);
 	found_merge_insert_method = (uint) (tmp >= 0 ? tmp : 0);
       }
       continue;		/* Skip comments */
@@ -89,9 +85,9 @@ MYRG_INFO *myrg_open(const char *name, int mode, int handle_locking)
 
     if (!has_path(buff))
     {
-      VOID(strmake(name_buff+dir_length,buff,
-                   sizeof(name_buff)-1-dir_length));
-      VOID(cleanup_dirname(buff,name_buff));
+      (void) strmake(name_buff+dir_length,buff,
+                   sizeof(name_buff)-1-dir_length);
+      (void) cleanup_dirname(buff,name_buff);
     }
     else
       fn_format(buff, buff, "", "", 0);
@@ -169,13 +165,14 @@ MYRG_INFO *myrg_open(const char *name, int mode, int handle_locking)
   m_info->last_used_table=m_info->open_tables;
   m_info->children_attached= TRUE;
 
-  VOID(my_close(fd,MYF(0)));
+  (void) mysql_file_close(fd, MYF(0));
   end_io_cache(&file);
-  VOID(pthread_mutex_init(&m_info->mutex, MY_MUTEX_INIT_FAST));
+  mysql_mutex_init(rg_key_mutex_MYRG_INFO_mutex,
+                   &m_info->mutex, MY_MUTEX_INIT_FAST);
   m_info->open_list.data=(void*) m_info;
-  pthread_mutex_lock(&THR_LOCK_open);
+  mysql_mutex_lock(&THR_LOCK_open);
   myrg_open_list=list_add(myrg_open_list,&m_info->open_list);
-  pthread_mutex_unlock(&THR_LOCK_open);
+  mysql_mutex_unlock(&THR_LOCK_open);
   DBUG_RETURN(m_info);
 
 bad_children:
@@ -186,13 +183,13 @@ err:
   case 3:
     while (files)
       (void) mi_close(m_info->open_tables[--files].table);
-    my_free((char*) m_info,MYF(0));
+    my_free(m_info);
     /* Fall through */
   case 2:
     end_io_cache(&file);
     /* Fall through */
   case 1:
-    VOID(my_close(fd,MYF(0)));
+    (void) mysql_file_close(fd, MYF(0));
   }
   my_errno=save_errno;
   DBUG_RETURN (NULL);
@@ -241,9 +238,11 @@ MYRG_INFO *myrg_parent_open(const char *parent_name,
   bzero((char*) &file_cache, sizeof(file_cache));
 
   /* Open MERGE meta file. */
-  if ((fd= my_open(fn_format(parent_name_buff, parent_name, "", MYRG_NAME_EXT,
-                             MY_UNPACK_FILENAME|MY_APPEND_EXT),
-                   O_RDONLY | O_SHARE, MYF(0))) < 0)
+  if ((fd= mysql_file_open(rg_key_file_MRG,
+                           fn_format(parent_name_buff, parent_name,
+                                     "", MYRG_NAME_EXT,
+                                     MY_UNPACK_FILENAME|MY_APPEND_EXT),
+                           O_RDONLY | O_SHARE, MYF(0))) < 0)
     goto err; /* purecov: inspected */
   errpos= 1;
 
@@ -272,7 +271,7 @@ MYRG_INFO *myrg_parent_open(const char *parent_name,
       {
         /* Compare buffer with global methods list: merge_insert_method. */
         insert_method= find_type(child_name_buff + 15,
-                                 &merge_insert_method, 2);
+                                 &merge_insert_method, FIND_TYPE_BASIC);
       }
       continue;
     }
@@ -318,13 +317,14 @@ MYRG_INFO *myrg_parent_open(const char *parent_name,
   }
 
   end_io_cache(&file_cache);
-  VOID(my_close(fd, MYF(0)));
-  VOID(pthread_mutex_init(&m_info->mutex, MY_MUTEX_INIT_FAST));
+  (void) mysql_file_close(fd, MYF(0));
+  mysql_mutex_init(rg_key_mutex_MYRG_INFO_mutex,
+                   &m_info->mutex, MY_MUTEX_INIT_FAST);
 
   m_info->open_list.data= (void*) m_info;
-  pthread_mutex_lock(&THR_LOCK_open);
+  mysql_mutex_lock(&THR_LOCK_open);
   myrg_open_list= list_add(myrg_open_list, &m_info->open_list);
-  pthread_mutex_unlock(&THR_LOCK_open);
+  mysql_mutex_unlock(&THR_LOCK_open);
 
   DBUG_RETURN(m_info);
 
@@ -333,13 +333,13 @@ MYRG_INFO *myrg_parent_open(const char *parent_name,
   save_errno= my_errno;
   switch (errpos) {
   case 3:
-    my_free((char*) m_info, MYF(0));
+    my_free(m_info);
     /* Fall through */
   case 2:
     end_io_cache(&file_cache);
     /* Fall through */
   case 1:
-    VOID(my_close(fd, MYF(0)));
+    (void) mysql_file_close(fd, MYF(0));
   }
   my_errno= save_errno;
   DBUG_RETURN (NULL);
@@ -385,6 +385,7 @@ int myrg_attach_children(MYRG_INFO *m_info, int handle_locking,
   uint       UNINIT_VAR(key_parts);
   uint       min_keys;
   my_bool    bad_children= FALSE;
+  my_bool    first_child= TRUE;
   DBUG_ENTER("myrg_attach_children");
   DBUG_PRINT("myrg", ("handle_locking: %d", handle_locking));
 
@@ -395,26 +396,36 @@ int myrg_attach_children(MYRG_INFO *m_info, int handle_locking,
     'open_tables' has all the pointers to the children. Use of a mutex
     here and in ha_myisammrg::store_lock() forces consistent data.
   */
-  pthread_mutex_lock(&m_info->mutex);
+  mysql_mutex_lock(&m_info->mutex);
   errpos= 0;
   file_offset= 0;
   min_keys= 0;
-  child_nr= 0;
-  while ((myisam= (*callback)(callback_param)))
+  for (child_nr= 0; child_nr < m_info->tables; child_nr++)
   {
+    if (! (myisam= (*callback)(callback_param)))
+    {
+      if (handle_locking & HA_OPEN_FOR_REPAIR)
+      {
+        /* An appropriate error should've been already pushed by callback. */
+        bad_children= TRUE;
+        continue;
+      }
+      goto bad_children;
+    }
+
     DBUG_PRINT("myrg", ("child_nr: %u  table: '%s'",
                         child_nr, myisam->filename));
-    DBUG_ASSERT(child_nr < m_info->tables);
 
     /* Special handling when the first child is attached. */
-    if (!child_nr)
+    if (first_child)
     {
+      first_child= FALSE;
       m_info->reclength= myisam->s->base.reclength;
       min_keys=  myisam->s->base.keys;
       key_parts= myisam->s->base.key_parts;
       if (*need_compat_check && m_info->rec_per_key_part)
       {
-        my_free((char *) m_info->rec_per_key_part, MYF(0));
+        my_free(m_info->rec_per_key_part);
         m_info->rec_per_key_part= NULL;
       }
       if (!m_info->rec_per_key_part)
@@ -456,14 +467,11 @@ int myrg_attach_children(MYRG_INFO *m_info, int handle_locking,
     for (idx= 0; idx < key_parts; idx++)
       m_info->rec_per_key_part[idx]+= (myisam->s->state.rec_per_key_part[idx] /
                                        m_info->tables);
-    child_nr++;
   }
 
   if (bad_children)
     goto bad_children;
-  /* Note: callback() resets my_errno, so it is safe to check it here */
-  if (my_errno == HA_ERR_WRONG_MRG_TABLE_DEF)
-    goto err;
+
   if (sizeof(my_off_t) == 4 && file_offset > (ulonglong) (ulong) ~0L)
   {
     my_errno= HA_ERR_RECORD_FILE_FULL;
@@ -474,7 +482,7 @@ int myrg_attach_children(MYRG_INFO *m_info, int handle_locking,
   m_info->keys= min_keys;
   m_info->last_used_table= m_info->open_tables;
   m_info->children_attached= TRUE;
-  pthread_mutex_unlock(&m_info->mutex);
+  mysql_mutex_unlock(&m_info->mutex);
   DBUG_RETURN(0);
 
 bad_children:
@@ -483,10 +491,10 @@ err:
   save_errno= my_errno;
   switch (errpos) {
   case 1:
-    my_free((char*) m_info->rec_per_key_part, MYF(0));
+    my_free(m_info->rec_per_key_part);
     m_info->rec_per_key_part= NULL;
   }
-  pthread_mutex_unlock(&m_info->mutex);
+  mysql_mutex_unlock(&m_info->mutex);
   my_errno= save_errno;
   DBUG_RETURN(1);
 }
@@ -509,7 +517,7 @@ int myrg_detach_children(MYRG_INFO *m_info)
 {
   DBUG_ENTER("myrg_detach_children");
   /* For symmetry with myrg_attach_children() we use the mutex here. */
-  pthread_mutex_lock(&m_info->mutex);
+  mysql_mutex_lock(&m_info->mutex);
   if (m_info->tables)
   {
     /* Do not attach/detach an empty child list. */
@@ -520,7 +528,7 @@ int myrg_detach_children(MYRG_INFO *m_info)
   m_info->del= 0;
   m_info->data_file_length= 0;
   m_info->options= 0;
-  pthread_mutex_unlock(&m_info->mutex);
+  mysql_mutex_unlock(&m_info->mutex);
   DBUG_RETURN(0);
 }
 
