@@ -1,4 +1,4 @@
-/* Copyright (c) 2002, 2013, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,8 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include <my_global.h>
 #include "m_string.h"
@@ -305,14 +304,13 @@ void my_hash_sort_simple(CHARSET_INFO *cs,
 			 ulong *nr1, ulong *nr2)
 {
   register uchar *sort_order=cs->sort_order;
-  const uchar *end= key + len;
+  const uchar *end;
   
   /*
     Remove end space. We have to do this to be able to compare
     'A ' and 'A' as identical
   */
-  while (end > key && end[-1] == ' ')
-    end--;
+  end= skip_trailing_space(key, len);
   
   for (; key < (uchar*) end ; key++)
   {
@@ -337,10 +335,6 @@ long my_strntol_8bit(CHARSET_INFO *cs,
   int overflow;
 
   *err= 0;				/* Initialize error indicator */
-#ifdef NOT_USED
-  if (base < 0 || base == 1 || base > 36)
-    base = 10;
-#endif
 
   s = nptr;
   e = nptr+l;
@@ -366,29 +360,6 @@ long my_strntol_8bit(CHARSET_INFO *cs,
   else
     negative = 0;
 
-#ifdef NOT_USED
-  if (base == 16 && s[0] == '0' && (s[1]=='X' || s[1]=='x'))
-    s += 2;
-#endif
-
-#ifdef NOT_USED
-  if (base == 0)
-  {
-    if (*s == '0')
-    {
-      if (s[1]=='X' || s[1]=='x')
-      {
-	s += 2;
-	base = 16;
-      }
-      else
-	base = 8;
-    }
-    else
-      base = 10;
-  }
-#endif
-
   save = s;
   cutoff = ((uint32)~0L) / (uint32) base;
   cutlim = (uint) (((uint32)~0L) % (uint32) base);
@@ -460,10 +431,6 @@ ulong my_strntoul_8bit(CHARSET_INFO *cs,
   int overflow;
 
   *err= 0;				/* Initialize error indicator */
-#ifdef NOT_USED
-  if (base < 0 || base == 1 || base > 36)
-    base = 10;
-#endif
 
   s = nptr;
   e = nptr+l;
@@ -488,29 +455,6 @@ ulong my_strntoul_8bit(CHARSET_INFO *cs,
   else
     negative = 0;
 
-#ifdef NOT_USED
-  if (base == 16 && s[0] == '0' && (s[1]=='X' || s[1]=='x'))
-    s += 2;
-#endif
-
-#ifdef NOT_USED
-  if (base == 0)
-  {
-    if (*s == '0')
-    {
-      if (s[1]=='X' || s[1]=='x')
-      {
-	s += 2;
-	base = 16;
-      }
-      else
-	base = 8;
-    }
-    else
-      base = 10;
-  }
-#endif
-
   save = s;
   cutoff = ((uint32)~0L) / (uint32) base;
   cutlim = (uint) (((uint32)~0L) % (uint32) base);
@@ -573,10 +517,6 @@ longlong my_strntoll_8bit(CHARSET_INFO *cs __attribute__((unused)),
   int overflow;
 
   *err= 0;				/* Initialize error indicator */
-#ifdef NOT_USED
-  if (base < 0 || base == 1 || base > 36)
-    base = 10;
-#endif
 
   s = nptr;
   e = nptr+l;
@@ -601,29 +541,6 @@ longlong my_strntoll_8bit(CHARSET_INFO *cs __attribute__((unused)),
   else
     negative = 0;
 
-#ifdef NOT_USED
-  if (base == 16 && s[0] == '0' && (s[1]=='X'|| s[1]=='x'))
-    s += 2;
-#endif
-
-#ifdef NOT_USED
-  if (base == 0)
-  {
-    if (*s == '0')
-    {
-      if (s[1]=='X' || s[1]=='x')
-      {
-	s += 2;
-	base = 16;
-      }
-      else
-	base = 8;
-    }
-    else
-      base = 10;
-  }
-#endif
-
   save = s;
 
   cutoff = (~(ulonglong) 0) / (unsigned long int) base;
@@ -696,10 +613,6 @@ ulonglong my_strntoull_8bit(CHARSET_INFO *cs,
   int overflow;
 
   *err= 0;				/* Initialize error indicator */
-#ifdef NOT_USED
-  if (base < 0 || base == 1 || base > 36)
-    base = 10;
-#endif
 
   s = nptr;
   e = nptr+l;
@@ -724,29 +637,6 @@ ulonglong my_strntoull_8bit(CHARSET_INFO *cs,
   else
     negative = 0;
 
-#ifdef NOT_USED
-  if (base == 16 && s[0] == '0' && (s[1]=='X' || s[1]=='x'))
-    s += 2;
-#endif
-
-#ifdef NOT_USED
-  if (base == 0)
-  {
-    if (*s == '0')
-    {
-      if (s[1]=='X' || s[1]=='x')
-      {
-	s += 2;
-	base = 16;
-      }
-      else
-	base = 8;
-    }
-    else
-      base = 10;
-  }
-#endif
-
   save = s;
 
   cutoff = (~(ulonglong) 0) / (unsigned long int) base;
@@ -951,16 +841,13 @@ cnv:
 #define INC_PTR(cs,A,B) (A)++
 
 
-static
-int my_wildcmp_8bit_impl(CHARSET_INFO *cs,
-                         const char *str,const char *str_end,
-                         const char *wildstr,const char *wildend,
-                         int escape, int w_one, int w_many, int recurse_level)
+int my_wildcmp_8bit(CHARSET_INFO *cs,
+		    const char *str,const char *str_end,
+		    const char *wildstr,const char *wildend,
+		    int escape, int w_one, int w_many)
 {
   int result= -1;			/* Not found, using wildcards */
 
-  if (my_string_stack_guard && my_string_stack_guard(recurse_level))
-    return 1;
   while (wildstr != wildend)
   {
     while (*wildstr != w_many && *wildstr != w_one)
@@ -1020,9 +907,8 @@ int my_wildcmp_8bit_impl(CHARSET_INFO *cs,
 	  str++;
 	if (str++ == str_end) return(-1);
 	{
-	  int tmp=my_wildcmp_8bit_impl(cs,str,str_end,
-                                       wildstr,wildend,escape,w_one,
-                                       w_many, recurse_level+1);
+	  int tmp=my_wildcmp_8bit(cs,str,str_end,wildstr,wildend,escape,w_one,
+				  w_many);
 	  if (tmp <= 0)
 	    return(tmp);
 	}
@@ -1033,16 +919,6 @@ int my_wildcmp_8bit_impl(CHARSET_INFO *cs,
   return(str != str_end ? 1 : 0);
 }
 
-int my_wildcmp_8bit(CHARSET_INFO *cs,
-                    const char *str,const char *str_end,
-                    const char *wildstr,const char *wildend,
-                    int escape, int w_one, int w_many)
-{
-  return my_wildcmp_8bit_impl(cs, str, str_end,
-                              wildstr, wildend,
-                              escape, w_one, w_many, 1);
-}
-
 
 /*
 ** Calculate min_str and max_str that ranges a LIKE string.
@@ -1180,9 +1056,8 @@ size_t my_well_formed_len_8bit(CHARSET_INFO *cs __attribute__((unused)),
 size_t my_lengthsp_8bit(CHARSET_INFO *cs __attribute__((unused)),
                         const char *ptr, size_t length)
 {
-  const char *end= ptr+length;
-  while (end > ptr && end[-1] == ' ')
-    end--;
+  const char *end;
+  end= (const char *) skip_trailing_space((const uchar *)ptr, length);
   return (size_t) (end-ptr);
 }
 
@@ -1402,19 +1277,6 @@ int my_mb_ctype_8bit(CHARSET_INFO *cs, int *ctype,
 }
 
 
-#undef  ULONGLONG_MAX
-/*
-  Needed under MetroWerks Compiler, since MetroWerks compiler does not
-  properly handle a constant expression containing a mod operator
-*/
-#if defined(__NETWARE__) && defined(__MWERKS__)
-static ulonglong ulonglong_max= ~(ulonglong) 0;
-#define ULONGLONG_MAX ulonglong_max
-#else
-#define ULONGLONG_MAX           (~(ulonglong) 0)
-#endif /* __NETWARE__ && __MWERKS__ */
-
-    
 #define CUTOFF  (ULONGLONG_MAX / 10)
 #define CUTLIM  (ULONGLONG_MAX % 10)
 #define DIGITS_IN_ULONGLONG 20
