@@ -1,4 +1,5 @@
-/* Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000-2007 MySQL AB, 2008, 2009 Sun Microsystems, Inc.
+   Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,8 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include <my_global.h>
 #include <m_ctype.h>
@@ -40,8 +40,6 @@
   
 */
 
-int (*my_string_stack_guard)(int)= NULL;
-
 static char *mstr(char *str,const char *src,size_t l1,size_t l2)
 {
   l1= l1<l2 ? l1 : l2;
@@ -77,6 +75,7 @@ struct my_cs_file_section_st
 #define	_CS_DIFF1	19
 #define	_CS_DIFF2	20
 #define	_CS_DIFF3	21
+#define	_CS_IDENTICAL	22
 
 
 static struct my_cs_file_section_st sec[] =
@@ -111,6 +110,7 @@ static struct my_cs_file_section_st sec[] =
   {_CS_DIFF1,		"charsets/charset/collation/rules/p"},
   {_CS_DIFF2,		"charsets/charset/collation/rules/s"},
   {_CS_DIFF3,		"charsets/charset/collation/rules/t"},
+  {_CS_IDENTICAL,	"charsets/charset/collation/rules/i"},
   {0,	NULL}
 };
 
@@ -272,6 +272,7 @@ static int cs_value(MY_XML_PARSER *st,const char *attr, size_t len)
   case _CS_DIFF1:
   case _CS_DIFF2:
   case _CS_DIFF3:
+  case _CS_IDENTICAL:
     {
       /*
         Convert collation description from
@@ -279,7 +280,7 @@ static int cs_value(MY_XML_PARSER *st,const char *attr, size_t len)
         into ICU Collation Customization expression.
       */
       char arg[16];
-      const char *cmd[]= {"&","<","<<","<<<"};
+      const char *cmd[]= {"&","<","<<","<<<","="};
       i->cs.tailoring= i->tailoring;
       mstr(arg,attr,len,sizeof(arg)-1);
       if (i->tailoring_length + 20 < sizeof(i->tailoring))
@@ -408,3 +409,23 @@ my_charset_is_8bit_pure_ascii(CHARSET_INFO *cs)
   }
   return 1;
 }
+
+
+/*
+  Shared function between conf_to_src and mysys.
+  Check if a 8bit character set is compatible with
+  ascii on the range 0x00..0x7F.
+*/
+my_bool
+my_charset_is_ascii_compatible(CHARSET_INFO *cs)
+{
+  uint i;
+  if (!cs->tab_to_uni)
+    return 1;
+  for (i= 0; i < 128; i++)
+  {
+    if (cs->tab_to_uni[i] != i)
+      return 0;
+  }
+  return 1;
+}
